﻿/*==============================================================================

   Name     : FMEServerRESTAPIdemo.cs

   System   : FME Server REST API

   Language : C#

   Purpose  : FMEServerAPIDemo sample application

   Author               Date            Changes made
   ------------------   ------------    -------------------------------
   Keith Huang          May 28, 2012    Original Definition
   Keith Huang          Jun 18, 2012    Method names set to upper camel
                                        case; connection authentication
   Matheson Mawhinney   Nov 21, 2014    Updated to REST API V2

                  Copyright (c) 1994 - 2014, Safe Software Inc.
                              All Rights Reserved

   The entire risk as to the results and performance of the software,
   supporting text and other information contained in this file
   (collectively called the "Software") is with the user.  Although
   Safe Software Incorporated has used considerable efforts in preparing
   the Software, Safe Software Incorporated does not warrant the
   accuracy or completeness of the Software. In no event will Safe Software
   Incorporated be liable for damages, including loss of profits or
   consequential damages, arising out of the use of the Software.

==============================================================================*/

// Requires JSON.Net library (Newtonsoft.Json.Linq)
using System;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;
using System.Net;
using Newtonsoft.Json.Linq;
using System.Net.Http;


namespace FMEServerRESTAPI_dotnet
{
   public class FMEServerRESTAPIDemo
   {
      private string userId_ = "";
      private string password_ = "";
      private string host_ = "";
      private string port_ = "";
      private string token_ = "";
      private string baseURL_ = "";

      //****************************UTILITY FUNCTIONS****************************

      //=========================================================================
      //
      private string GetResponseString(HttpWebResponse response)
      {
         Stream stream = response.GetResponseStream();
         StreamReader reader = new StreamReader(stream, Encoding.UTF8);
         string responseString = reader.ReadToEnd();
         return responseString;
      }

      //=========================================================================
      //
      private string ResponseStr(byte[] byteArray) // Converts a byte[] to a string object
      {
         return Encoding.UTF8.GetString(byteArray);
      }

      //=========================================================================
      //
      private string GetInput(string promptMessage) // Outputs a prompt and stores user input
      {
         Console.WriteLine(promptMessage + "\t");
         return Console.ReadLine();
      }

      //=========================================================================
      //
      public FMEServerRESTAPIDemo(string host, string port) // Constructor
      {
         host_ = host;
         port_ = port;
         baseURL_ = "http://" + host + ":" + port + "/fmerest/v2/";
         GetConnectionInfo();
      }

      //=========================================================================
      //
      public void GetConnectionInfo() // Gets information necessary to make a server connection
      {
         Console.WriteLine("Please enter server connection credentials.");
         userId_ = GetInput("UserId");

         if (userId_ != "")
         {
            password_ = GetInput("Password");
            // We must first generate a security token for authentication purposes
            string fmeUrl = "http://" + host_ + ":" + port_ + "/fmetoken/service/generate?user=" + userId_ + "&password=" +
               password_ + "&expiration=1&timeunit=hour";

            WebClient client = new WebClient();

            token_ = client.UploadString(fmeUrl, "GET");
         }
      }

      //=========================================================================
      //
      public void Execute() // Gets action and performs it
      {
         DisplayMainMenu();
         try
         {
            int choice = GetMenuChoice();
            ExecuteMenuChoice(choice);
         }
         catch (FormatException)
         {
            Console.Error.WriteLine("Invalid input!");
         }
         catch (Exception e)
         {
            Console.Error.WriteLine(e.Message);
         }
      }

      //=========================================================================
      //
      private void DisplayMainMenu()
      {
         Console.Error.WriteLine("\n\n" + "===FME Server REST API Demo===\n"
               + "1. List available repositories\n" + "2. Add a repository\n"
               + "3. Remove a repository\n" + "4. List available workspaces\n"
               + "5. Add a workspace\n" + "6. Update a workspace\n"
               + "7. Get a workspace\n" + "8. Remove a workspace\n"
               + "9. List resources of a workspace\n" + "10. Add a resource\n"
               + "11. Update a resource\n" + "12. Get a resource\n"
               + "13. Remove a resource\n" + "14. List available services\n"
               + "15. Add a service\n" + "16. Update a service\n"
               + "17. Remove a service\n" + "18. Run a workspace\n"
               + "19. List published parameters of a workspace\n"
               + "20. List jobs\n" + "21. View a job\n" + "0. Quit\n"
               + "\nSELECT: ");
      }

      //=========================================================================
      //
      private int GetMenuChoice()
      {
         return Int32.Parse(Console.ReadLine());
      }

      //=========================================================================
      //
      private void ExecuteMenuChoice(int choice)
      {
         switch (choice)
         {
            case 1:
               ListRepositories();
               break;
            case 2:
               AddRepository();
               break;
            case 3:
               RemoveRepository();
               break;
            case 4:
               ListWorkspaces();
               break;
            case 5:
               AddWorkspace();
               break;
            case 6:
               UpdateWorkspace();
               break;
            case 7:
               GetWorkspace();
               break;
            case 8:
               RemoveWorkspace();
               break;
            case 9:
               ListResources();
               break;
            case 10:
               AddResource();
               break;
            case 11:
               UpdateResource();
               break;
            case 12:
               GetResource();
               break;
            case 13:
               RemoveResource();
               break;
            case 14:
               ListServices();
               break;
            case 15:
               AddService();
               break;
            case 16:
               UpdateService();
               break;
            case 17:
               RemoveService();
               break;
            case 18:
               RunWorkspace();
               break;
            case 19:
               ListParameters();
               break;
            case 20:
               ListJobs();
               break;
            case 21:
               ViewJob();
               break;
            case 0:
               Environment.Exit(0);
               break;
            default:
               Console.Error.WriteLine("Invalid option!");
               return;
         }
         Console.WriteLine("\nThe operation was a success");
      }

      //=========================================================================
      //
      private void ListRepositories()
      {
         string fmeUrl = baseURL_ + "repositories?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "GET";
            response = (HttpWebResponse)request.GetResponse();

            string responseString = GetResponseString(response);

            JArray json = JArray.Parse(responseString);
            Console.WriteLine("=============================================");
            foreach (JToken token in json)
            {
               Console.WriteLine("Name: " + token.Value<String>("name"));
               Console.WriteLine("Description: " + token.Value<String>("description"));
               Console.WriteLine("=============================================");
            }
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if (response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void AddRepository()
      {
         string name = GetInput("Please enter the name for the new repository:");
         string description = GetInput("Please enter the description for the new repository:");

         string fmeUrl = baseURL_ + "repositories?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "POST";

            string postString = String.Format("name={0}&description={1}", name, description);
            StreamWriter requestWriter = new StreamWriter(request.GetRequestStream());
            requestWriter.Write(postString);
            requestWriter.Close();

            response = (HttpWebResponse)request.GetResponse();
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void RemoveRepository()
      {
         string name = GetInput("Please enter the name of the repository to be removed:");

         string fmeUrl = baseURL_ + "repositories/" + name + "?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "DELETE";

            response = (HttpWebResponse)request.GetResponse();
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void ListWorkspaces()
      {
         string name = GetInput("Please enter the name of the repository to show workspaces for:");
         string fmeUrl = baseURL_ + "repositories/" + name + "/items?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "GET";
            response = (HttpWebResponse)request.GetResponse();

            string responseString = GetResponseString(response);

            JArray json = JArray.Parse(responseString);
            Console.WriteLine("=============================================");
            foreach(JToken token in json)
            {
               Console.WriteLine("Name: " + token.Value<String>("name"));
               Console.WriteLine("=============================================");
            }
         }
         catch(WebException e)
         {
            throw new Exception(e.Message);
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      // File upload with HttpWebRequest non-trivial. HttpClient in .NET 4.5 also a good option for file upload
      private void AddWorkspace()
      {
         string repositoryName = GetInput("Please enter the name of the repository to put the new workspace: ");
         string workspaceFilePath = GetInput("Please enter the local file path of the new workspace to add: ");

         string fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items?token=" + token_;

         WebClient client = new WebClient();

         try
         {
            client.UploadFile(fmeUrl, "POST", workspaceFilePath);
         }
         catch (WebException e)
         {
            throw e;
         }
      }


      //=========================================================================
      //
      // File upload with HttpWebRequest non-trivial. HttpClient in .NET 4.5 also a good option for file upload
      private void UpdateWorkspace()
      {
         string repositoryName = GetInput("Please enter the name of the repository containing the workspace to update: ");
         string workspaceFilePath = GetInput("Please enter the local file path of the new workspace: ");
         string workspaceToReplace = GetInput("Please enter the name of the workspace to update: ");

         string fmeurl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceToReplace + "?token=" + token_;

         WebClient client = new WebClient();

         try
         {
            client.UploadFile(fmeurl, "PUT", workspaceFilePath);
         }
         catch (WebException e)
         {
            throw e;
         }
      }

      //=========================================================================
      //
      private void GetWorkspace() // Gets workspaces,custom transformers, and custom formats
      {
         string repositoryName = GetInput("Please enter the repository name for the workspace to get:");
         string workspaceName = GetInput("Please enter the workspace name (with extension):");
         string workspaceFilePath = GetInput("Please enter the local file path for the workspace to be stored:");

         string fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "GET";
            request.Accept = "application/octet-stream";

            response = (HttpWebResponse)request.GetResponse();

            string responseString = GetResponseString(response);

            using (StreamWriter file = new StreamWriter(workspaceFilePath))
            {
               file.Write(responseString);
            }
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void RemoveWorkspace()
      {
         string repositoryName = GetInput("Please enter the repository name for the workspace to remove:");
         string workspaceName = GetInput("Please enter the name for the workspace to remove:");

         string fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "DELETE";

            response = (HttpWebResponse)request.GetResponse();
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void ListResources()
      {
         string repositoryName = GetInput("Please enter the repository name for the resources:");
         string workspaceName = GetInput("Please enter the the workspace name for the resources:");

         string fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "/resources?token=" + token_;


         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "GET";
            response = (HttpWebResponse)request.GetResponse();

            string responseString = GetResponseString(response);

            JArray json = JArray.Parse(responseString);
            Console.WriteLine("=============================================");
            foreach(JToken token in json)
            {
               Console.WriteLine("Name: " + token.Value<String>("name"));
               Console.WriteLine("=============================================");
            }
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void AddResource()
      {
         string repositoryName = GetInput("Please enter the name of the repository for the new resource:");
         string workspaceName = GetInput("Please enter the name of the workspace for the new new resource:");
         string resourceFilePath = GetInput("Please enter the local file path for the new resource to add:");

         string fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "/resources?token=" + token_;

         WebClient client = new WebClient();

         try
         {
            client.UploadFile(fmeUrl, "POST", resourceFilePath);
         }
         catch(WebException e)
         {
            throw e;
         }
      }

      //=========================================================================
      //
      private void UpdateResource()
      {
         string repositoryName = GetInput("Please enter the repository name for the resource to update:");
         string workspaceName = GetInput("Please enter the workspace name for the resource to update:");
         string existingResource = GetInput("Please enter the name of the resource to be replaced:");
         string resourceFilePath = GetInput("Please enter the file path of the resource to update (with extensions):");

         string fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "/resources/" +
            existingResource + "?token=" + token_;

         WebClient client = new WebClient();

         try
         {
            client.UploadFile(fmeUrl, "PUT", resourceFilePath);
         }
         catch (WebException e)
         {
            throw e;
         }
      }

      //=========================================================================
      //
      private void GetResource()
      {
         string repositoryName = GetInput("Please enter the repository name for the resource to get:");
         string workspaceName = GetInput("Please enter the workspace name for the resource to get:");
         string resourceName = GetInput("Please enter the resource name for the resource to get:");
         string resourceFilePath = GetInput("Please enter the local file path for the resource to be stored:");

         string fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName +
            "/resources/" + resourceName + "?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "GET";
            request.Accept = "application/octet-stream";

            response = (HttpWebResponse)request.GetResponse();

            string responseString = GetResponseString(response);

            using(StreamWriter file = new StreamWriter(resourceFilePath))
            {
               file.Write(responseString);
            }
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void RemoveResource()
      {
         string repositoryName = GetInput("Please enter the repository name for the resource to be removed:");
         string workspaceName = GetInput("Please enter the workspace name for the resource to be removed:");
         string resourceName = GetInput("Please enter the resource name (with extensions) to be removed:");

         string fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "/resources/"
            + resourceName + "?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "DELETE";

            response = (HttpWebResponse)request.GetResponse();
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void ListServices()
      {
         string fmeUrl = baseURL_ + "services?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "GET";
            response = (HttpWebResponse)request.GetResponse();

            string responseString = GetResponseString(response);

            JArray json = JArray.Parse(responseString);
            Console.WriteLine("=============================================");
            foreach(JToken token in json)
            {
               Console.WriteLine("Name: " + token.Value<String>("name"));
               Console.WriteLine("=============================================");
            }
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void AddService()
      {
         string name = GetInput("Please enter the name of the new service:");
         string urlPattern = GetInput("Please enter the URL pattern of the new service:");
         string displayName = GetInput("Please enter the display name of the new service:");
         string description = GetInput("Please enter the description of the new service:");

         string fmeUrl = baseURL_ + "services?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "POST";

            string postString = String.Format("name={0}&url={1}&displayName={2}&description={3}",
               name, urlPattern, displayName, description);
            StreamWriter requestWriter = new StreamWriter(request.GetRequestStream());
            requestWriter.Write(postString);
            requestWriter.Close();

            response = (HttpWebResponse)request.GetResponse();
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void UpdateService()
      {
         string oldName = GetInput("Please enter the name of the existing service to update:");
         string newName = GetInput("Please enter the new name of the service:");
         string urlPattern = GetInput("Please enter the new URL pattern of the service:");
         string displayName = GetInput("Please enter the new display name of the service:");
         string description = GetInput("Please enter the new description of the service:");

         string fmeUrl = baseURL_ + "services/" + oldName + "?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "PUT";

            string putString = String.Format("name={0}&displayName={1}&description={2}&url={3}",
               newName, displayName, description, urlPattern);
            StreamWriter requestWriter = new StreamWriter(request.GetRequestStream());
            requestWriter.Write(putString);
            requestWriter.Close();

            response = (HttpWebResponse)request.GetResponse();
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void RemoveService()
      {
         string serviceName = GetInput("Please enter the name of the service to be removed:");

         string fmeUrl = baseURL_ + "services/" + serviceName + "?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "DELETE";

            response = (HttpWebResponse)request.GetResponse();
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void RunWorkspace()
      {
         string repositoryName = GetInput("Please enter the name for the repository:");
         string workspaceName = GetInput("Please enter the name for the workspace to run:");

         string fmeUrl = baseURL_ + "transformations/commands/transact/" + repositoryName + "/" +
            workspaceName + "?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "POST";
            request.ContentType = "application/json";

            string postString = "{}";
            StreamWriter requestWriter = new StreamWriter(request.GetRequestStream());
            requestWriter.Write(postString);
            requestWriter.Close();

            response = (HttpWebResponse)request.GetResponse();

            string responseString = GetResponseString(response);

            JToken token = JToken.Parse(responseString);
            Console.WriteLine("=============================================");
            Console.WriteLine("ID: " + token.Value<String>("id"));
            Console.WriteLine("Status: " + token.Value<String>("status"));
            Console.WriteLine("Status Message: " + token.Value<String>("statusMessage"));
            Console.WriteLine("Time Finished: " + token.Value<String>("timeFinished"));
            Console.WriteLine("=============================================");
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      public void ListParameters()
      {
         string repositoryName = GetInput("Please enter the repository name for the workspace:");
         string workspaceName = GetInput("Please enter the name of the workspace to view the parameters of:");

         string fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName +
            "/parameters?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "GET";

            response = (HttpWebResponse)request.GetResponse();

            string responseString = GetResponseString(response);

            JArray json = JArray.Parse(responseString);
            Console.WriteLine("=============================================");
            foreach(JToken token in json)
            {
               Console.WriteLine("Name: " + token.Value<String>("name"));
               Console.WriteLine("Description: " + token.Value<String>("description"));
               Console.WriteLine("=============================================");
            }
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private void ListJobs()
      {
         // We would like to see the IDs of all jobs listed
         // So, we perform the same action for each job type
         string[] categories = {
               "running",
               "completed",
               "queued"
         };

         foreach (string category in categories)
         {
            Console.WriteLine(category);
            string fmeUrl = baseURL_ + "transformations/jobs/" + category + "?token=" + token_;

            HttpWebRequest request;
            HttpWebResponse response = null;
            try
            {
               request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
               request.Method = "GET";
               response = (HttpWebResponse)request.GetResponse();

               Stream stream = response.GetResponseStream();
               StreamReader reader = new StreamReader(stream, Encoding.UTF8);
               String responseString = reader.ReadToEnd();

               JArray json = JArray.Parse(responseString);

               Console.WriteLine("=============================================");
               foreach (JToken token in json)
               {
                  string id = token.Value<String>("id");
                  string status = token.Value<String>("status");
                  string timeFinished = token.Value<String>("timeFinished");
                  Console.WriteLine("ID: " + id);
                  Console.WriteLine("Status: " + status);
                  Console.WriteLine("Time Finished: " + timeFinished);
                  Console.WriteLine("=============================================");
               }
               Console.WriteLine();

            }
            catch(WebException e)
            {
               throw e;
            }
            catch (Exception e)
            {
               Console.Error.WriteLine(e.Message);
               continue; // If one call is bad, the other jobs should still be listed
            }
            finally
            {
               if(response != null)
               {
                  response.Close();
               }
            }
         }
      }

      //=========================================================================
      //
      private void ViewJob()
      {
         string jobID = GetInput("Please enter the id of the job to view:");

         string fmeUrl = baseURL_ + "transformations/jobs/id/" + jobID + "?token=" + token_;

         HttpWebRequest request;
         HttpWebResponse response = null;

         try
         {
            request = (HttpWebRequest)HttpWebRequest.Create(fmeUrl);
            request.Method = "GET";

            response = (HttpWebResponse)request.GetResponse();

            string responseString = GetResponseString(response);

            JToken json = JToken.Parse(responseString);

            Console.WriteLine("=============================================");
            Console.WriteLine("ID: " + json.Value<String>("id"));
            Console.WriteLine("Status: " + json.Value<String>("status"));
            Console.WriteLine("Time Started: " + json.Value<String>("timeStarted"));
            Console.WriteLine("Time Finished: " + json.Value<String>("timeFinished"));
            Console.WriteLine("=============================================");
         }
         catch(WebException e)
         {
            throw e;
         }
         finally
         {
            if(response != null)
            {
               response.Close();
            }
         }
      }

      //=========================================================================
      //
      private static void ArgError()
      {
         Console.Error.WriteLine("USAGE:\n\tFMEServerAPIDemo <host> <port>");
         Environment.Exit(0);
      }

      public static void Main(string[] args)
      {
         string host = null;
         string port = null;

         // Extract the arguments from the command line
         if (args.Length != 2)
         {
            ArgError();
         }
         else
         {
            host = args[0];
            port = args[1];
         }
         try
         {
            FMEServerRESTAPIDemo demo = new FMEServerRESTAPIDemo(host, port);

            while (true)
            {
               demo.Execute();
            }
         }
         catch (Exception e)
         {
            Console.WriteLine(e.Message);
         }
            Console.ReadLine();
        }
   }
}
