/*==============================================================================

   Name     : FMEServerRESTAPIdemo.java

   System   : FME Server REST API

   Language : Java

   Purpose  : FMEServerAPIDemo sample application

   Author               Date            Changes made
   ------------------   ------------    -------------------------------
   Keith Huang          May 28, 2012    Original Definition
   Matheson Mawhinney   Nov 21, 2014    Updated to REST V2 API

                  Copyright (c) 1994 - 2014, Safe Software Inc.
                              All Rights Reserved

   The entire risk as to the results and performance of the software,
   supporting text and other information contained in this file
   (collectively called the "Software") is with the user.  Although
   Safe Software Incorporated has used considerable efforts in preparing
   the Software, Safe Software Incorporated does not warrant the
   accuracy or completeness of the Software. In no event will Safe Software
   Incorporated be liable for damages, including loss of profits or
   consequential damages, arising out of the use of the Software.

==============================================================================*/

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

/** Apache HttpClient 4.3 is required **/
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.*;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONObject;

public class FMEServerRESTAPIDemo
{

   private CloseableHttpClient  client_         = null;
   // Used to read content from the user
   private BufferedReader        br_            = null;
   // Security token for authentication
   private String                token_         = null;

   private String                host_          = null;
   private int                   port_          = 0;

   private String                baseURL_       = null;

   public FMEServerRESTAPIDemo(String host, int port) throws Exception
   {
      br_ = new BufferedReader(new InputStreamReader(System.in));

      host_ = host;
      port_ = port;

      client_ = HttpClients.createDefault();

      baseURL_ = "http://" + host + ":" + port + "/fmerest/v2/";

      try
      {
         getConnectionInfo();
      }
      catch(IOException e)
      {
         System.err.println(e.getMessage());
      }
   }

   public void getConnectionInfo() throws Exception
   {
      System.out.println("Please enter server connection credentials.");
      System.out.print("UserId:");
      String userId = br_.readLine();

      if (!userId.equals("")) {
         System.out.print("Password:");
         String password = br_.readLine();

         // We must first generate a security token for authentication purposes
         String fmeUrl = "http://" + host_ + ":" + port_ + "/fmetoken/service/generate?user=" + userId + "&password=" +
                 password + "&expiration=1&timeunit=hour";
         CloseableHttpResponse response = null;

         try
         {
            HttpGet method = new HttpGet(fmeUrl);
            response = client_.execute(method);

            ResponseHandler<String> handler = new BasicResponseHandler();
            String body = handler.handleResponse(response);

            int code = response.getStatusLine().getStatusCode();
            if (code == 200)
            {
               token_ = body;
            }
            else
            {
               throw new Exception("Authentication failed");
            }
         }
         finally
         {
            if (response != null)
            {
               response.close();
            }
         }
      }
   }

   public void execute()
   {
      displayMainMenu();
      try
      {
         int choice = getMenuChoice();
         executeMenuChoice(choice);
      }
      catch(NumberFormatException e)
      {
         System.err.println("Invalid input!");
      }
      catch(Exception e)
      {
         System.err.println(e.getMessage());
      }
   }
//
   private void displayMainMenu()
   {
      System.out.print("\n\n" + "===FME Server REST API Demo===\n"
              + "1. List available repositories\n" + "2. Add a repository\n"
              + "3. Remove a repository\n" + "4. List available workspaces\n"
              + "5. Add a workspace\n" + "6. Update a workspace\n"
              + "7. Get a workspace\n" + "8. Remove a workspace\n"
              + "9. List resources of a workspace\n" + "10. Add a resource\n"
              + "11. Update a resource\n" + "12. Get a resource\n"
              + "13. Remove a resource\n" + "14. List available services\n"
              + "15. Add a service\n" + "16. Update a service\n"
              + "17. Remove a service\n" + "18. Run a workspace\n"
              + "19. List published parameters of a workspace\n"
              + "20. List jobs\n" + "21. View a job\n"
              + "0. Quit\n"
              + "\nSELECT: ");
   }

   private int getMenuChoice() throws NumberFormatException, IOException
   {
      return Integer.parseInt(br_.readLine());
   }

   private void executeMenuChoice(int choice) throws Exception
   {
      switch(choice)
      {
         case 1:
            listRepositories();
            break;
         case 2:
            addRepository();
            break;
         case 3:
            removeRepository();
            break;
         case 4:
            listWorkspaces();
            break;
         case 5:
            addWorkspace();
            break;
         case 6:
            updateWorkspace();
            break;
         case 7:
            getWorkspace();
            break;
         case 8:
            removeWorkspace();
            break;
         case 9:
            listResources();
            break;
         case 10:
            addResource();
            break;
         case 11:
            updateResource();
            break;
         case 12:
            getResource();
            break;
         case 13:
            removeResource();
            break;
         case 14:
            listServices();
            break;
         case 15:
            addService();
            break;
         case 16:
            updateService();
            break;
         case 17:
            removeService();
            break;
         case 18:
            runWorkspace();
            break;
         case 19:
            listParameters();
            break;
         case 20:
            listJobs();
            break;
         case 21:
            viewJob();
            break;
         case 0:
            System.exit(0);
         default:
            System.err.println("Invalid option!");
            return;
      }
      System.out.println("\nThe operation was a success");
   }

   private void listRepositories() throws Exception
   {
      String fmeUrl = baseURL_ + "repositories?token=" + token_;
      CloseableHttpResponse response = null;
      try
      {
         HttpGet method = new HttpGet(fmeUrl);
         response = client_.execute(method);
         ResponseHandler<String> handler = new BasicResponseHandler();
         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }

         JSONArray json = new JSONArray(handler.handleResponse(response));

         System.out.println("=============================================");

         for (int i = 0; i < json.length(); i++)
         {
            System.out.println("Name: " + json.getJSONObject(i).get("name"));
            System.out.println("Description: " + json.getJSONObject(i).get("description"));
            System.out.println("=============================================");
         }
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void addRepository() throws Exception
   {
      String name = getInput("Please enter the name for the new repository:");
      String description = getInput("Please enter the description for the new repository:");

      String fmeUrl = baseURL_ + "repositories?token=" + token_;
      CloseableHttpResponse response = null;

      try
      {
         HttpPost postMethod = new HttpPost(fmeUrl);

         List<NameValuePair> params = new ArrayList<NameValuePair>();
         params.add(new BasicNameValuePair("name", name));
         params.add(new BasicNameValuePair("description", description));
         postMethod.setEntity(new UrlEncodedFormEntity(params));

         response = client_.execute(postMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 201)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void removeRepository() throws Exception
   {
      String name = getInput("Please enter the name of the repository to remove:");

      String fmeUrl = baseURL_ + "repositories/" + name + "?token=" + token_;
      CloseableHttpResponse response = null;

      try
      {
         HttpDelete deleteMethod = new HttpDelete(fmeUrl);
         response = client_.execute(deleteMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 204)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void listWorkspaces() throws Exception
   {
      String name = getInput("Please enter the name of the repository to show the workspaces for:");

      String fmeUrl = baseURL_ + "repositories/" + name + "/items?token=" + token_;
      CloseableHttpResponse response = null;

      try
      {
         HttpGet getMethod = new HttpGet(fmeUrl);
         response = client_.execute(getMethod);

         ResponseHandler<String> handler = new BasicResponseHandler();
         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }

         JSONArray json = new JSONArray(handler.handleResponse(response));

         System.out.println("=============================================");
         for (int i = 0; i < json.length(); i++)
         {
            System.out.println("Name: " + json.getJSONObject(i).get("name"));
            System.out.println("=============================================");
         }
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void addWorkspace() throws Exception
   {
      String repositoryName = getInput("Please enter the name of the repository to put the new workspace:");
      String workspaceFilePath = getInput("Please enter the local file path of the new workspace to add:");

      File file = new File(workspaceFilePath);
      String workspaceName = file.getName();
      HttpEntity entity = null;
      CloseableHttpResponse response = null;

      String fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items?token=" + token_;

      try
      {
         HttpPost postMethod = new HttpPost(fmeUrl);

         FileBody body = new FileBody(file);
         entity = MultipartEntityBuilder.create().addPart("file", body).build();
         postMethod.setEntity(entity);
         postMethod.addHeader("Content-Disposition", "attachment; filename=\"" + workspaceName + "\"");
         response = client_.execute(postMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code >= 300)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }
      }
      finally
      {
         EntityUtils.consume(entity);
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void updateWorkspace() throws Exception
   {
      String repositoryName = getInput("Please enter the name of the repository containing the workspace to update:");
      String workspaceFilePath = getInput("Please enter the local file path of the new workspace:");
      String workspaceToReplace = getInput("Please enter the name of the workspace to update:");

      String fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceToReplace + "?token=" + token_;

      File file = new File(workspaceFilePath);
      HttpEntity entity = null;
      CloseableHttpResponse response = null;

      try
      {
         HttpPut putMethod = new HttpPut(fmeUrl);

         FileBody body = new FileBody(file);
         entity = MultipartEntityBuilder.create().addPart("file", body).build();
         putMethod.setEntity(entity);
         putMethod.addHeader("Content-Type", "application/octet-stream");
         response = client_.execute(putMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }
      }
      finally
      {
         EntityUtils.consume(entity);
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void getWorkspace() throws Exception
   {
      String repositoryName = getInput("Please enter the name of the repository containing the workspace to get:");
      String workspaceName = getInput("Please enter the name of the workspace to get:");
      String workspaceFilePath = getInput("Please enter the local file path for the workspace to be saved at:");

      String fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "?token=" + token_;
      FileOutputStream file = null;
      PrintStream fileStream = null;
      HttpEntity entity = null;
      CloseableHttpResponse response = null;

      try
      {
         HttpGet getMethod = new HttpGet(fmeUrl);
         getMethod.addHeader("Accept", "application/octet-stream");

         file = new FileOutputStream(workspaceFilePath);
         fileStream = new PrintStream(file);
         response = client_.execute(getMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }

         entity = response.getEntity();
         byte[] responseBody = EntityUtils.toByteArray(entity);
         fileStream.println(new String(responseBody));
      }
      finally
      {
         EntityUtils.consume(entity);
         if(fileStream != null)
         {
            fileStream.close();
         }
         if(file != null)
         {
            file.close();
         }
         if(response != null)
         {
            response.close();
         }
      }
   }

   private void removeWorkspace() throws Exception
   {
      String repositoryName = getInput("Please enter the repository name for the workspace to remove:");
      String workspaceName = getInput("Please enter the name of the workspace to remove:");

      String fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "?token=" + token_;

      CloseableHttpResponse response = null;

      try
      {
         HttpDelete deleteMethod = new HttpDelete(fmeUrl);
         response = client_.execute(deleteMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 204)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void listResources() throws Exception
   {
      String repositoryName = getInput("Please enter the name of the repository containing the workspace:");
      String workspaceName = getInput("Please enter the name of the workspace to get resources from:");

      String fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "/resources?token=" + token_;

      CloseableHttpResponse response = null;

      try
      {
         HttpGet getMethod = new HttpGet(fmeUrl);
         response = client_.execute(getMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }

         ResponseHandler<String> handler = new BasicResponseHandler();
         JSONArray json = new JSONArray(handler.handleResponse(response));

         System.out.println("=============================================");
         for(int i = 0; i < json.length(); i++)
         {
            System.out.println("Name: " + json.getJSONObject(i).get("name"));
            System.out.println("=============================================");
         }
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void addResource() throws Exception
   {
      String repositoryName = getInput("Please enter the name of the repository for the new resource:");
      String workspaceName = getInput("Please enter the name of the workspace for the new resource:");
      String resourceFilePath = getInput("Please enter the local file path for the new resource to add:");

      String fmeUrl = baseURL_  + "repositories/" + repositoryName + "/items/" + workspaceName + "/resources?token=" + token_;

      File file = new File(resourceFilePath);
      String resourceName = file.getName();
      HttpEntity entity = null;
      CloseableHttpResponse response = null;

      try
      {
         HttpPost postMethod = new HttpPost(fmeUrl);

         FileBody fileBody = new FileBody(file);
         entity = MultipartEntityBuilder.create().addPart("file", fileBody).build();
         postMethod.setEntity(entity);
         postMethod.addHeader("Content-Type", "application/octet-stream");
         postMethod.addHeader("Content-Disposition", "attachment; filename=\"" + resourceName + "\"");
         response = client_.execute(postMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 201)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }
      }
      finally
      {
         EntityUtils.consume(entity);
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void updateResource() throws Exception
   {
      String repositoryName = getInput("Please enter the repository name for the resource to update:");
      String workspaceName = getInput("Please enter the workspace name for the resource to update:");
      String existingResource = getInput("Please enter the name of the resource to be replaced:");
      String resourceFilePath = getInput("Please enter the local file path for the resource to update:");

      File file = new File(resourceFilePath);
      HttpEntity entity = null;
      CloseableHttpResponse response = null;

      String fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "/resources/" +
              existingResource + "?token=" + token_;

      try
      {
         HttpPut putMethod = new HttpPut(fmeUrl);

         FileBody fileBody = new FileBody(file);
         entity = MultipartEntityBuilder.create().addPart("file", fileBody).build();
         putMethod.setEntity(entity);
         putMethod.addHeader("Content-Type", "application/octet-stream");
         response = client_.execute(putMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }
      }
      finally
      {
         EntityUtils.consume(entity);
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void getResource() throws Exception
   {
      String repositoryName = getInput("Please enter the repository name for the resource to get:");
      String workspaceName = getInput("Please enter the workspace name for the resource to get:");
      String resourceName = getInput("Please enter the resource name for the resource to get:");
      String resourceFilePath = getInput("Please enter the local file path for the resource to get:");

      String fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "/resources/" +
              resourceName + "?token=" + token_;

      FileOutputStream file = null;
      PrintStream printStream = null;
      HttpEntity entity = null;
      CloseableHttpResponse response = null;

      try
      {
         HttpGet getMethod = new HttpGet(fmeUrl);
         getMethod.addHeader("Accept", "application/octet-stream");

         file = new FileOutputStream(resourceFilePath);
         printStream = new PrintStream(file);
         response = client_.execute(getMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }

         entity = response.getEntity();
         byte[] responseBody = EntityUtils.toByteArray(entity);
         printStream.println(new String(responseBody));
      }
      finally
      {
         EntityUtils.consume(entity);
         if (printStream != null)
         {
            printStream.close();
         }
         if (file != null)
         {
            file.close();
         }
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void removeResource() throws Exception
   {
      String repositoryName = getInput("Please enter the repository name for the resource to remove:");
      String workspaceName = getInput("Please enter the workspace name for the resource to remove:");
      String resourceName = getInput("Please enter the resource name for the resource to remove:");

      String fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "/resources/" +
              resourceName + "?token=" + token_;

      CloseableHttpResponse response = null;

      try
      {
         HttpDelete deleteMethod = new HttpDelete(fmeUrl);
         response = client_.execute(deleteMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 204)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void listServices() throws Exception
   {
      String fmeUrl = baseURL_ + "services?token=" + token_;

      CloseableHttpResponse response = null;

      try
      {
         HttpGet getMethod = new HttpGet(fmeUrl);
         response = client_.execute(getMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }

         ResponseHandler<String> handler = new BasicResponseHandler();
         JSONArray json = new JSONArray(handler.handleResponse(response));

         System.out.println("=============================================");
         for (int i = 0; i < json.length(); i++)
         {
            System.out.println("Name: " + json.getJSONObject(i).get("name"));
            System.out.println("=============================================");
         }
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void addService() throws Exception
   {
      String name = getInput("Please enter the name of the new service:");
      String urlPattern = getInput("Please enter the URL pattern of the new service:");
      String displayName = getInput("Please enter the display name of the new service:");
      String description = getInput("Please enter the description of the new service:");

      String fmeUrl = baseURL_ + "services?token=" + token_;

      CloseableHttpResponse response = null;

      try
      {
         HttpPost postMethod = new HttpPost(fmeUrl);

         List<NameValuePair> params = new ArrayList<NameValuePair>();
         params.add(new BasicNameValuePair("description", description));
         params.add(new BasicNameValuePair("displayName", displayName));
         params.add(new BasicNameValuePair("name", name));
         params.add(new BasicNameValuePair("url", urlPattern));
         postMethod.setEntity(new UrlEncodedFormEntity(params));

         response = client_.execute(postMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 201)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }
      }
      finally
      {
         if (response != null) {
            response.close();
         }
      }
   }

   private void updateService() throws Exception
   {
      String oldName = getInput("Please enter the name of the existing service to update:");
      String newName = getInput("Please enter the new name of the service:");
      String urlPattern = getInput("Please enter the new URL pattern of the service:");
      String displayName = getInput("Please enter the new display name of the service:");
      String description = getInput("Please enter the new description of the service:");

      String fmeUrl = baseURL_ + "services/" + oldName + "?token=" + token_;

      CloseableHttpResponse response = null;

      try
      {
         HttpPut putMethod = new HttpPut(fmeUrl);

         List<NameValuePair> params = new ArrayList<NameValuePair>();
         params.add(new BasicNameValuePair("description", description));
         params.add(new BasicNameValuePair("displayName", displayName));
         params.add(new BasicNameValuePair("name", newName));
         params.add(new BasicNameValuePair("url", urlPattern));
         putMethod.setEntity(new UrlEncodedFormEntity(params));

         response = client_.execute(putMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void removeService() throws Exception
   {
      String name = getInput("Please enter the name of the service to remove:");

      String fmeUrl = baseURL_ + "services/" + name + "?token=" + token_;

      CloseableHttpResponse response = null;

      try
      {
         HttpDelete deleteMethod = new HttpDelete(fmeUrl);
         response = client_.execute(deleteMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 204)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void runWorkspace() throws Exception
   {
      String repositoryName = getInput("Please enter the repository name for the workspace to run:");
      String workspaceName = getInput("Please enter the name of the workspace to run:");

      String fmeUrl = baseURL_ + "transformations/commands/transact/" + repositoryName + "/" +
              workspaceName + "?token=" + token_;

      CloseableHttpResponse response = null;

      try
      {
         HttpPost postMethod = new HttpPost(fmeUrl);
         postMethod.addHeader("Content-Type", "application/json");

         String params = "{}";

         postMethod.setEntity(new StringEntity(params));
         response = client_.execute(postMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }

         ResponseHandler<String> handler = new BasicResponseHandler();
         JSONObject json = new JSONObject(handler.handleResponse(response));

         System.out.println("=============================================");
         System.out.println("ID: " + json.get("id"));
         System.out.println("Status: " + json.get("status"));
         System.out.println("Status Message: " + json.get("statusMessage"));
         System.out.println("Time Finished: " + json.get("timeFinished"));
         System.out.println("=============================================");
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   public void listParameters() throws Exception {

      String repositoryName = getInput("Please enter the repository name for the workspace to view the parameters of:");
      String workspaceName = getInput("Please enter the name of the workspace to view the parameters of:");

      String fmeUrl = baseURL_ + "repositories/" + repositoryName + "/items/" + workspaceName + "/parameters?token=" + token_;

      CloseableHttpResponse response = null;

      try
      {
         HttpGet getMethod = new HttpGet(fmeUrl);
         response = client_.execute(getMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }

         ResponseHandler<String> handler = new BasicResponseHandler();
         JSONArray json = new JSONArray(handler.handleResponse(response));

         System.out.println("=============================================");
         for(int i = 0; i < json.length(); i++)
         {
            System.out.println("Name: " + json.getJSONObject(i).get("name"));
            System.out.println("Description: " + json.getJSONObject(i).get("description"));
            System.out.println("=============================================");
         }
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private void listJobs() throws Exception
   {
      // We would like to see the IDs of all jobs listed
      // So, we perform the same action for each job type
      String[] categories = {
              "running",
              "completed",
              "queued"
      };

      CloseableHttpResponse response = null;

      for (String category : categories) {
         String fmeUrl = baseURL_ + "transformations/jobs/" + category + "?token=" + token_;
         System.out.println(category);
         try
         {
            HttpGet getMethod = new HttpGet(fmeUrl);
            response = client_.execute(getMethod);

            int code = response.getStatusLine().getStatusCode();
            if (code != 200)
            {
               throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
            }

            ResponseHandler<String> handler = new BasicResponseHandler();
            JSONArray json = new JSONArray(handler.handleResponse(response));

            System.out.println("=============================================");
            for (int i = 0; i < json.length(); i++)
            {
               System.out.println("ID: " + json.getJSONObject(i).get("id"));
               System.out.println("Status: " + json.getJSONObject(i).get("status"));
               System.out.println("Time Finished: " + json.getJSONObject(i).get("timeFinished"));
               System.out.println("=============================================");
            }
         }
         finally
         {
            if (response != null)
            {
               response.close();
            }
         }
      }
   }

   private void viewJob() throws Exception
   {
      String id = getInput("Please enter the id of the job to view: ");

      String fmeUrl = baseURL_ + "transformations/jobs/id/" + id + "?token=" + token_;

      CloseableHttpResponse response = null;

      try
      {
         HttpGet getMethod = new HttpGet(fmeUrl);
         response = client_.execute(getMethod);

         int code = response.getStatusLine().getStatusCode();
         if (code != 200)
         {
            throw new Exception(code + ": " + response.getStatusLine().getReasonPhrase());
         }

         ResponseHandler<String> handler = new BasicResponseHandler();

         JSONObject json = new JSONObject(handler.handleResponse(response));
         System.out.println("=============================================");
         System.out.println("ID: " + json.get("id"));
         System.out.println("Status: " + json.get("status"));
         System.out.println("Time Finished: " + json.get("timeFinished"));
         System.out.println("=============================================");
      }
      finally
      {
         if (response != null)
         {
            response.close();
         }
      }
   }

   private String getInput(String promptMessage) throws Exception
   {
      System.out.print(promptMessage + "\t");
      return br_.readLine();
   }

   /**
    * This program accepts two parameters. First parameter is the FME Server
    * host. ie. localhost Second parameter is the FME Server port. ie. 7071
    */
   public static void main(String args[]) throws Exception
   {
      String host = null;
      int port = 0;

      // Extract the arguments from the command line
      if(args.length != 2)
      {
         argError();
      }
      else
      {
         host = args[0];
         try
         {
            port = Integer.parseInt(args[1]);
         }
         catch(NumberFormatException nfe)
         {
            argError();
         }
      }

      FMEServerRESTAPIDemo demo = new FMEServerRESTAPIDemo(host, port);

      while(true)
      {
         demo.execute();
      }
   }

   private static void argError()
   {
      System.err.println("USAGE:\n\tFMEServerRESTAPIDemo <host> <port>");
      System.exit(-1);
   }
}
