# ==============================================================================
#
#   Name     : FMEServerRESTAPIdemo.py
#
#   System   : FME Server REST API
#
#   Language : Python
#
#   Purpose  : FMEServerAPIDemo sample application
#
#   Author               Date            Changes made
#   ------------------   ------------    -------------------------------
#   Matheson Mawhinney   Nov 21, 2014    Original Definition
#
#            Copyright (c) 1994 - 2014, Safe Software Inc.
#                        All Rights Reserved
#
#   The entire risk as to the results and performance of the software,
#   supporting text and other information contained in this file
#   (collectively called the "Software") is with the user.  Although
#   Safe Software Incorporated has used considerable efforts in preparing
#   the Software, Safe Software Incorporated does not warrant the
#   accuracy or completeness of the Software. In no event will Safe Software
#   Incorporated be liable for damages, including loss of profits or
#   consequential damages, arising out of the use of the Software.
#
# ==============================================================================

# Requires installing the Requests library: http://docs.python-requests.org/en/latest/
import sys
import requests

host_ = ""
port_ = ""
userId_ = ""
password_ = ""
token_ = ""
base_url_ = ""


def main(argv):
    if len(sys.argv) != 3:
        arg_error()

    global host_
    host_ = argv[1]
    global port_
    port_ = argv[2]

    global base_url_
    base_url_ = "http://" + host_ + ":" + port_ + "/fmerest/v2/"
    get_connection_info()

    while True:
        execute()


def get_connection_info():
    print "Please enter server connection credentials."
    global userId_
    userId_ = raw_input("UserId: ")

    if userId_ != "":
        global password_
        password_ = raw_input("Password: ")

        fme_url = "http://" + host_ + ":" + port_ + "/fmetoken/service/generate?user=" \
                  + userId_ + "&password=" + password_ + "&expiration=1&timeunit=hour"

        request = requests.get(fme_url)
        global token_
        token_ = request.text


def execute():
    try:
        display_main_menu()
        choice = get_menu_choice()
        execute_menu_choice(choice)
    except Exception as detail:
        print detail


def get_menu_choice():
    return int(raw_input())


def execute_menu_choice(choice):
    if choice == 1:
        list_repositories()
    elif choice == 2:
        add_repository()
    elif choice == 3:
        remove_repository()
    elif choice == 4:
        list_workspaces()
    elif choice == 5:
        add_workspace()
    elif choice == 6:
        update_workspace()
    elif choice == 7:
        get_workspace()
    elif choice == 8:
        remove_workspace()
    elif choice == 9:
        list_resources()
    elif choice == 10:
        add_resource()
    elif choice == 11:
        update_resource()
    elif choice == 12:
        get_resource()
    elif choice == 13:
        remove_resource()
    elif choice == 14:
        list_services()
    elif choice == 15:
        add_service()
    elif choice == 16:
        update_service()
    elif choice == 17:
        remove_service()
    elif choice == 18:
        run_workspace()
    elif choice == 19:
        list_parameters()
    elif choice == 20:
        list_jobs()
    elif choice == 21:
        view_job()
    elif choice == 0:
        sys.exit(0)
    else:
        print "Invalid option!"
        return
    print "\nThe operation was a success"


def display_main_menu():
    print "\n\n" + "===FME Server REST API Demo===\n" \
          + "1. List available repositories\n" + "2. Add a repository\n" \
          + "3. Remove a repository\n" + "4. List available workspaces\n" \
          + "5. Add a workspace\n" + "6. Update a workspace\n" \
          + "7. Get a workspace\n" + "8. Remove a workspace\n" \
          + "9. List resources of a workspace\n" + "10. Add a resource\n" \
          + "11. Update a resource\n" + "12. Get a resource\n" \
          + "13. Remove a resource\n" + "14. List available services\n" \
          + "15. Add a service\n" + "16. Update a service\n" \
          + "17. Remove a service\n" + "18. Run a workspace\n" \
          + "19. List published parameters of a workspace\n" \
          + "20. List jobs\n" + "21. View a job\n" + "0. Quit\n" \
          + "\nSELECT: "


def arg_error():
    print "Usage:\nRESTDemo.py <host> <port>"
    sys.exit(0)


def list_repositories():
    fme_url = base_url_ + "repositories?token=" + token_
    response = requests.get(fme_url)
    if response.status_code != 200:
        raise Exception(str(response.status_code) + ": " + response.reason)

    json_content = response.json()
    print "============================================="
    for x in range(0, len(json_content)):
        print "Name: " + json_content[x]["name"]
        print "Description: " + json_content[x]["description"]
        print "============================================="


def add_repository():
    name = get_input("Please enter the name for the new repository:")
    description = get_input("Please enter the description for the new repository:")

    fme_url = base_url_ + "repositories?token=" + token_

    data = {'name': name, 'description': description}

    response = requests.post(fme_url, data=data)
    if response.status_code != 201:
        raise Exception(str(response.status_code) + ": " + response.reason)


def remove_repository():
    name = get_input("Please enter the name of the repository to remove:")
    fme_url = base_url_ + "repositories/" + name + "?token=" + token_
    response = requests.delete(fme_url)
    if response.status_code != 204:
        raise Exception(str(response.status_code) + ": " + response.reason)


def list_workspaces():
    name = get_input("PLease enter the name of the repository to show workspaces for:")
    fme_url = base_url_ + "repositories/" + name + "/items?token=" + token_

    response = requests.get(fme_url)
    if response.status_code != 200:
        raise Exception(str(response.status_code) + ": " + response.reason)

    json_content = response.json()
    print "============================================="
    for x in range(0, len(json_content)):
        print "Name: " + json_content[x]['name']
        print "============================================="


def add_workspace():
    repository_name = get_input("Please enter the name of the repository to put the new workspace in:")
    workspace_file_path = get_input("Please enter the local file path of the new workspace to add:")

    path_parts = workspace_file_path.split("\\")
    file_name = path_parts[len(path_parts) - 1]

    fme_url = base_url_ + "repositories/" + repository_name + "/items?token=" + token_
    files = {'file': open(workspace_file_path, 'rb')}
    headers = {'Content-Disposition': 'attachment; filename=\"' + file_name + '\"',
               'Content-Type': 'application/octet-stream'}
    response = requests.post(fme_url, files=files, headers=headers)
    if response.status_code != 201:
        raise Exception(str(response.status_code) + ": " + response.reason)


def update_workspace():
    repository_name = get_input("Please enter the name of the repository containing the workspace to update:")
    workspace_file_path = get_input("Please enter the local file path of the new workspace:")
    workspace_to_replace = get_input("Please enter the name of the workspace to update (including extension):")

    fme_url = base_url_ + "repositories/" + repository_name + "/items/" + workspace_to_replace + "?token=" + token_

    path_parts = workspace_file_path.split("\\")
    file_name = path_parts[len(path_parts) - 1]

    files = {'file': open(workspace_file_path, 'rb')}
    headers = {'Content-Disposition': 'attachment; filename=\"' + file_name + '\"',
               'Content-Type': 'application/octet-stream'}
    response = requests.put(fme_url, files=files, headers=headers)
    if response.status_code != 200:
        raise Exception(str(response.status_code) + ": " + response.reason)


def get_workspace():
    repository_name = get_input("Please enter the name of the repository containing the workspace to download:")
    workspace_name = get_input("Please enter the name of the workspace to download (including extension):")
    workspace_file_path = get_input("Please enter the local file path for the workspace to be saved at:")

    fme_url = base_url_ + "repositories/" + repository_name + "/items/" + workspace_name + "?token=" + token_

    headers = {'Accept': 'application/octet-stream'}

    response = requests.get(fme_url, headers=headers)
    if response.status_code != 200:
        raise Exception(str(response.status_code) + ": " + response.reason)

    response_body = response.content
    file_to_write = open(workspace_file_path, 'wb')
    file_to_write.write(response_body)


def remove_workspace():
    repository_name = get_input("Please enter the name of the repository containing the workspace to remove:")
    workspace_name = get_input("Please enter the name of the workspace to remove (including extension):")

    fme_url = base_url_ + "repositories/" + repository_name + "/items/" + workspace_name + "?token=" + token_

    response = requests.delete(fme_url)
    if response.status_code != 204:
        raise Exception(str(response.status_code) + ": " + response.reason)


def list_resources():
    repository_name = get_input("Please enter the name of the repository containing the workspace:")
    workspace_name = get_input("Please enter the name of the workspace to get resources from:")

    fme_url = base_url_ + "repositories/" + repository_name + "/items/" + workspace_name + "/resources?token=" + token_

    response = requests.get(fme_url)
    if response.status_code != 200:
        raise Exception(str(response.status_code) + ": " + response.reason)

    json_content = response.json()
    print "============================================="
    for x in range(0, len(json_content)):
        print "Name: " + json_content[x]['name']
        print "============================================="


def add_resource():
    repository_name = get_input("Please enter the name of the repository for the new resource:")
    workspace_name = get_input("Please enter the name of the workspace for the new resource:")
    resource_file_path = get_input("Please enter the local file path of the new resource to add:")

    fme_url = base_url_ + "repositories/" + repository_name + "/items/" + workspace_name + "/resources?token=" + token_

    path_parts = resource_file_path.split("\\")
    file_name = path_parts[len(path_parts) - 1]

    files = {'file': open(resource_file_path, 'rb')}
    headers = {'Content-Disposition': 'attachment; filename=\"' + file_name + '\"',
               'Content-Type': 'application/octet-stream'}
    response = requests.post(fme_url, files=files, headers=headers)
    if response.status_code != 201:
        raise Exception(str(response.status_code) + ": " + response.reason)


def update_resource():
    repository_name = get_input("Please enter the repository name for the resource to update:")
    workspace_name = get_input("Please enter the workspace name for the resource to update:")
    existing_resource = get_input("Please enter the name of the resource to be replaced (including extension):")
    resource_file_path = get_input("Please enter the local file path for the resource to update:")

    fme_url = base_url_ + "repositories/" + repository_name + "/items/" + workspace_name + "/resources/" \
        + existing_resource + "?token=" + token_

    files = {'file': open(resource_file_path, 'rb')}
    headers = {'Content-Disposition': 'attachment; filename=\"' + existing_resource + '\"',
               'Content-Type': 'application/octet-stream'}
    response = requests.put(fme_url, files=files, headers=headers)
    if response.status_code != 200:
        raise Exception(str(response.status_code) + ": " + response.reason)


def get_resource():
    repository_name = get_input("Please enter the repository name for the resource to download:")
    workspace_name = get_input("Please enter the workspace name for the resource to download:")
    resource_name = get_input("Please enter the resource name for the resource to download (including extension):")
    resource_file_path = get_input("Please enter the local file path for the resource to download:")

    fme_url = base_url_ + "repositories/" + repository_name + "/items/" + workspace_name + "/resources/" \
        + resource_name + "?token=" + token_

    headers = {'Accept': 'application/octet-stream'}

    response = requests.get(fme_url, headers=headers)
    if response.status_code != 200:
        raise Exception(str(response.status_code) + ": " + response.reason)

    response_body = response.content
    file_to_write = open(resource_file_path, 'wb')
    file_to_write.write(response_body)


def remove_resource():
    repository_name = get_input("Please enter the repository name for the resource to remove:")
    workspace_name = get_input("Please enter the workspace name for the resource to remove:")
    resource_name = get_input("Please enter the resource name for the resource to remove (including extension):")

    fme_url = base_url_ + "repositories/" + repository_name + "/items/" + workspace_name + "/resources/" \
        + resource_name + "?token=" + token_

    response = requests.delete(fme_url)
    if response.status_code != 204:
        raise Exception(str(response.status_code) + ": " + response.reason)


def list_services():
    fme_url = base_url_ + "services?token=" + token_

    response = requests.get(fme_url)
    if response.status_code != 200:
        raise Exception(str(response.status_code) + ": " + response.reason)

    json_content = response.json()
    print "============================================="
    for x in range(0, len(json_content)):
        print "Name: " + json_content[x]['name']
        print "============================================="


def add_service():
    name = get_input("Please enter the name of the new service:")
    url_pattern = get_input("Please enter the URL pattern of the new service:")
    display_name = get_input("Please enter the display name of the new service:")
    description = get_input("Please enter the description of the new service:")

    fme_url = base_url_ + "services?token=" + token_

    data = {'description': description, 'name': name, 'url': url_pattern, 'displayName': display_name}

    response = requests.post(fme_url, data=data)
    if response.status_code != 201:
        raise Exception(str(response.status_code) + ": " + response.reason)


def update_service():
    old_name = get_input("Please enter the name of the existing service to update:")
    new_name = get_input("Please enter the new name of the service:")
    url_pattern = get_input("Please enter the new URL pattern of the service:")
    display_name = get_input("Please enter the new display name of the service:")
    description = get_input("Please enter the new description of the service:")

    fme_url = base_url_ + "services/" + old_name + "?token=" + token_

    data = {'description': description, 'displayName': display_name, 'name': new_name, 'url': url_pattern}

    response = requests.put(fme_url, data=data)
    if response.status_code != 200:
        raise Exception(str(response.status_code) + ": " + response.reason)


def remove_service():
    name = get_input("PLease enter the name of the service to remove:")

    fme_url = base_url_ + "services/" + name + "?token=" + token_

    response = requests.delete(fme_url)
    if response.status_code != 204:
        raise Exception(str(response.status_code) + ": " + response.reason)


def run_workspace():
    repository_name = get_input("Please enter the repository name for the workspace to run:")
    workspace_name = get_input("Please enter the name of the workspace to run (including extension):")

    fme_url = base_url_ + "transformations/commands/transact/" + repository_name + "/" + workspace_name + "?token=" \
        + token_

    headers = {"Content-Type": "application/json"}
    params = "{}"
    response = requests.post(fme_url, data=params, headers=headers)
    if response.status_code != 200:
        raise Exception(str(response.status_code) + ": " + response.reason)

    json_content = response.json()
    print "============================================="
    print "ID: " + str(json_content['id'])
    print "Status: " + json_content['status']
    print "Status Message: " + json_content['statusMessage']
    print "Time Finished: " + json_content['timeFinished']
    print "============================================="


def list_parameters():
    repository_name = get_input("Please enter the repository name for the workspace to view the parameters of:")
    workspace_name = get_input("Please enter the name of the workspace to view parameters of (including extension):")

    fme_url = base_url_ + "repositories/" + repository_name + "/items/" + workspace_name + "/parameters?token=" + token_

    response = requests.get(fme_url)
    if response.status_code != 200:
        raise Exception(str(response.status_code) + ": " + response.reason)

    json_content = response.json()
    print "============================================="
    for x in range(0, len(json_content)):
        print "Name: " + json_content[x]['name']
        print "Description: " + json_content[x]['description']
        print "============================================="


def list_jobs():
    categories = ["running", "completed", "queued"]

    for category in categories:
        fme_url = base_url_ + "transformations/jobs/" + category + "?token=" + token_
        print category

        response = requests.get(fme_url)
        if response.status_code != 200:
            raise Exception(str(response.status_code) + ": " + response.reason)

        json_content = response.json()
        print "============================================="
        for x in range(0, len(json_content)):
            print "ID: " + str(json_content[x]['id'])
            print "Status: " + json_content[x]['status']
            print "Time Finished: " + json_content[x]['timeFinished']
            print "============================================="


def view_job():
    job_id = get_input("Please enter the id of the job to view:")
    fme_url = base_url_ + "transformations/jobs/id/" + job_id + "?token=" + token_

    response = requests.get(fme_url)
    if response.status_code != 200:
        raise Exception("Getting job failed")

    json_content = response.json()
    print "============================================="
    print "ID: " + str(json_content['id'])
    print "Status: " + json_content['status']
    print "Time Finished: " + json_content['timeFinished']
    print "============================================="


def get_input(prompt):
    return raw_input(prompt + "\t")


if __name__ == '__main__':
    main(sys.argv)











